package mediateca;

import java.util.*;
import java.io.*;

public class Mediateca
{
	Map<String, List<Media>> titoli, tipi;
	Map<String, Comparator<Media>> ordinamenti;
	BufferedReader stdin;

	public Mediateca()
	{
		titoli = new HashMap<String, List<Media>>();
		tipi = new HashMap<String, List<Media>>();
		ordinamenti = new HashMap<String, Comparator<Media>>();
	}

	public boolean leggiDati(String nomeFile)
	{
		File f = new File(nomeFile);
		FileReader fr = null;
		try
		{
			fr = new FileReader(f);
		}
		catch(FileNotFoundException fnfe)
		{
			fnfe.printStackTrace();
			return false;
		}

		BufferedReader br = new BufferedReader(fr);

		try
		{
			String line = null;
			while( (line = br.readLine()) != null)
			{
				Media m = null;
				if(line.equals("Dvd"))
				{
					m = new Dvd();
				}
				else if(line.equals("AudioCd"))
				{
					//m = new AudioCd();
				}
				else
				{
					System.err.println("Media di tipo <" + line + "> non gestito");
					continue;
				}
				if(m.populate(br))
				{
					List<Media> l = titoli.get(m.titolo);
					if(l == null)
					{
						l = new LinkedList<Media>();
						titoli.put(m.titolo, l);
					}
					l.add(m);

					l = tipi.get(line);
					if(l == null)
					{
						l = new LinkedList<Media>();
						tipi.put(line, l);
					}
					l.add(m);
				}
			}

			System.out.println(tipi);
			System.out.println(titoli);
		}
		catch(IOException ioe)
		{
			ioe.printStackTrace();
		}

		try
		{
			br.close();
		}
		catch(IOException ioe2)
		{
			ioe2.printStackTrace();
		}
		return true;
	}

	void tipo()
	{
		try
		{
			System.out.print("Che tipo di media " + tipi.keySet() + "? > ");
			String tipo = stdin.readLine();
			System.out.print("Che tipo di ordinamento " + ordinamenti.keySet() + "? > ");
			String ordinamento = stdin.readLine();

			if(tipo == null || ordinamento == null)
			{
				System.err.println("Input inconsistente");
				return;
			}

			Comparator<Media> comp = ordinamenti.get(ordinamento);
			List<Media> l = tipi.get(tipo);

			if(comp == null)
			{
				System.err.println("Ordinamento <" + ordinamento + "> sconosciuto");
				return;
			}

			if(l == null)
			{
				System.err.println("Nessun media di tipo <" + tipo + "> presente nel sistema");
				return;
			}

			Collections.sort(l, comp);
			System.out.println(l);
		}
		catch(IOException ioe)
		{
			ioe.printStackTrace();
		}
	}

	public static void main(String[] argv)
	{
		Mediateca mediateca = new Mediateca();
		mediateca.ordinamenti.put("titolo", new ComparaPerTitolo());

		int i;
		for(i=0;i<argv.length;i++)
		{
			mediateca.leggiDati(argv[i]);
		}

		mediateca.stdin = new BufferedReader(new InputStreamReader(System.in));

		while(true)
		{
			String cmd = null;
			System.out.print("[TIPO/TITOLO/ESCI] ? > ");
			try
			{
				cmd = mediateca.stdin.readLine();
			}
			catch(IOException ioe)
			{
				ioe.printStackTrace();
				return;
			}

			if(cmd == null || cmd.equals("ESCI"))
			{
				break;
			}
			else if(cmd.equals("TIPO"))
			{
				mediateca.tipo();
			}
			else if(cmd.equals("TITOLO"))
			{
				//titolo();
			}
			else
			{
				System.err.println("Comando <" + cmd + "> non riconosciuto");
			}
		}
	}
}
