package stack;

import dinamiche.Nodo;

public class StackDinamico<E>
implements Stack<E>
{
	Nodo<E> head;
	int size;

	/**
	*Creates an empty Stack
	*/
	public StackDinamico()
	{
		head = null;
		size = 0;
	}

//Method Summary
	/**
	*Tests if this stack is empty.
	*/
	public boolean empty()
	{
		return size == 0;
	}
	
	/**
	*Looks at the object at the top of this stack without removing it from the stack.
	*/
	public E peek()
	{
		if(empty())
		{
			throw new NoSuchElementException("Stack vuoto");
		}
		return head.getDato();
	}
	
	/**
	*Removes the object at the top of this stack and returns that object as the value of this function.
	*/
	public E pop()
	{
		E retval = peek();
		head = head.getNext();
		size--;
		return retval;
	}
	
	/**
	*Pushes an item onto the top of this stack.
	*/
	public E push(E item)
	{
		Nodo<E> tmp = new Nodo<E>(item, head);
		head = tmp;
		size++;
		return item;
	}
	
	/**
	*Returns the 1-based position where an object is on this stack.
	*/
	public int search(E o)
	{
		int retval = -1;
		int count = 1;
		Nodo<E> tmp = head;
		if(o == null)
		{
			while(tmp != null)
			{
				if(tmp.getDato() == null)
				{
					return count;
				}
				tmp = tmp.getNext();
				count++;
			}
		}
		else
		{
			while(tmp != null)
			{
				if(o.equals(tmp.getDato()))
				{
					return count;
				}
				tmp = tmp.getNext();
				count++;
			}
		}
		return retval;
	}

	/**
	*Returns the number of elements inside the Stack.
	*/
	public int howMany()
	{
		return size;
	}

	/**
	*Return a String representation of the Stack and its content.
	*/
	public String toString()
	{
		if(empty())
		{
			return "[]";
		}
		String retval = "[";
		Nodo<E> tmp = head;
		while(tmp != null && tmp.getNext() != null)
		{
			retval = retval.concat(tmp.getDato() + ", ");
			tmp = tmp.getNext();
		}
		retval = retval.concat(tmp.getDato() + "]");
		return retval;
	}

	public boolean equals(Object o)
	{
		/* Solo se so che confrontare oggetti disomogenei NON e' un errore
		 * logico
		 */
		/*
		if(!(o instanceof Stack))
		{
			return false;
		}
		StackDinamico s = (StackDinamico)o;
		*/

		/* Confrontare oggetti disomogenei E' un errore logico */
		StackDinamico s = (StackDinamico)o;

		if(howMany() != s.howMany())
		{
			return false;
		}

		Nodo tmp1, tmp2;
		tmp1 = head;
		tmp2 = s.head;
		while(tmp1 != null)
		{
			if(!tmp1.equals(tmp2))
			{
				return false;
			}
			tmp1 = tmp1.getNext();
			tmp2 = tmp2.getNext();
		}
		return true;
	}
}
