package terminal;

import java.io.*;
import java.util.*;

public class Terminal
{
	BufferedReader stdin;
	List<Transito>[] date;
	Map<String, List<Transito>> destinazioni, tipi, ditte;
	Comparator<Transito> comp;

	public Terminal()
	{
		date = (List<Transito>[])(new LinkedList[31]);
		destinazioni = new HashMap<String, List<Transito>>();
		tipi = new HashMap<String, List<Transito>>();
		ditte = null;

		stdin = new BufferedReader(new InputStreamReader(System.in));
		comp = new ComparaDataDitta();
	}

	public static void main(String[] argv)
	{
		(new Terminal()).esegui(argv);
	}

	void esegui(String[] args)
	{
		String cmd = null;
		while(true)
		{
			System.out.print("[LEGGI/TRANSITI/DESTINAZIONE/MERCE/ESCI]? > ");
			try
			{
				cmd = stdin.readLine();
			}
			catch(IOException ioe)
			{
				ioe.printStackTrace();
				System.exit(1);
			}

			if(cmd == null || cmd.equals("ESCI"))
			{
				//se vogliamo mantenere lo stato raggiunto questo e' il punto in
				//cui farlo.
				break;
			}
			else if(cmd.equals("LEGGI"))
			{
				leggi();
			}
			else if(cmd.equals("TRANSITI"))
			{
				transiti();
			}
			else if(cmd.equals("DESTINAZIONE"))
			{
				//destinazione();
			}
			else if(cmd.equals("MERCE"))
			{
				//merce();
			}
			else
			{
				System.err.println("Comando <" + cmd + "> non riconosciuto");
			}
		}
	}

	void transiti()
	{
		int data = -1;
		System.out.print("Data da visualizzare? > ");
		try
		{
			data = Integer.parseInt(stdin.readLine());
			if(data < 1 || data > 31)
			{
				throw new IllegalArgumentException(String.valueOf(data) + " out of range");
			}
		}
		catch(NumberFormatException nfe)
		{
			nfe.printStackTrace();
			return;
		}
		catch(IOException ioe)
		{
			ioe.printStackTrace();
			return;
		}
		catch(IllegalArgumentException iae)
		{
			iae.printStackTrace();
			return;
		}

		List<Transito> l = date[data-1];
		if(l == null)
		{
			System.out.println("Non ci sono stati transiti nel giorno " + data);
			return;
		}

		Collections.sort(l, comp);
		System.out.println(l);
	}

	void leggi()
	{
		String nomeFile = null;
		BufferedReader br = null;
		System.out.print("Nome file dati? > ");
		try
		{
			nomeFile = stdin.readLine();
			File f = new File(nomeFile);
			FileReader fr = new FileReader(f);
			br = new BufferedReader(fr);
		}
		catch(IOException ioe)
		{
			ioe.printStackTrace();
			return;
		}

		while(true)
		{
			Transito t = new Transito();
			if(!t.popola(br))
			{
				break;
			}

			List<Transito> l = date[t.data-1];
			if(l == null)
			{
				l = new LinkedList<Transito>();
				date[t.data-1] = l;
			}
			l.add(t);

			l = destinazioni.get(t.dest);
			if(l == null)
			{
				l = new LinkedList<Transito>();
				destinazioni.put(t.dest, l);
			}
			l.add(t);

			l = tipi.get(t.tipo);
			if(l == null)
			{
				l = new LinkedList<Transito>();
				tipi.put(t.tipo, l);
			}
			l.add(t);
		}

		try
		{
			br.close();
		}
		catch(IOException ioe)
		{
			ioe.printStackTrace();
		}

		//stampe di debug
		System.out.print("date: ");
		for(int i = 0; i< date.length;i++)
		{
			System.out.print("{" + i + ": " + date[i] + "}; ");
		}
		System.out.println("tipi: " + tipi);
		System.out.println("destinazioni: " + destinazioni);
	}
}
